@@ -236,10 +236,8 @@ default_alloc_buffer (GstBufferPool * pool, GstBuffer ** buffer,
 static gboolean
 mark_meta_pooled (GstBuffer * buffer, GstMeta ** meta, gpointer user_data)
 {
-  GstBufferPool *pool = user_data;
-
-  GST_DEBUG_OBJECT (pool, "marking meta %p as POOLED in buffer %p", *meta,
-      buffer);
+  GST_DEBUG_OBJECT (GST_BUFFER_POOL (user_data),
+      "marking meta %p as POOLED in buffer %p", *meta, buffer);
   GST_META_FLAG_SET (*meta, GST_META_FLAG_POOLED);
   GST_META_FLAG_SET (*meta, GST_META_FLAG_LOCKED);
 
